package de.w665.sharepulse.db.repo;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import com.rethinkdb.RethinkDB;
import com.rethinkdb.net.Connection;
import de.w665.sharepulse.db.RethinkDBConnector;
import de.w665.sharepulse.model.FileUpload;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.lang.reflect.Type;
import java.util.*;

@Repository
@RequiredArgsConstructor
public class FileUploadRepository {

    private final RethinkDB r;
    private final Connection connection;
    private final Gson gson;

    @Autowired
    public FileUploadRepository(RethinkDBConnector connector) {
        this.r = connector.getR();
        this.connection = connector.getConnection();

        this.gson = new GsonBuilder().create();
    }

    public void insertFileUpload(FileUpload fileUpload) {
        String json = gson.toJson(fileUpload);

        Type type = new TypeToken<Map<String, Object>>(){}.getType();
        Map<String, Object> map = gson.fromJson(json, type);

        long uploadDateTimestamp = fileUpload.getUploadDate().getTime();
        map.put("uploadDate", uploadDateTimestamp);

        r.db("sharepulse").table("file_uploads").insert(map).run(connection);
    }

    public Optional<FileUpload> retrieveFileUploadByFileId(String fileId) {
        try {
            FileUpload fileUpload = r.db("sharepulse").table("file_uploads")
                    .filter(r.hashMap("fileId", fileId))
                    .run(connection, FileUpload.class)
                    .next();
            return Optional.ofNullable(fileUpload);
        } catch (NoSuchElementException e) {
            return Optional.empty();
        }

    }

    public void updateFileUpload(FileUpload updatedFileUpload) {
        String json = gson.toJson(updatedFileUpload);

        Type type = new TypeToken<Map<String, Object>>(){}.getType();
        Map<String, Object> map = gson.fromJson(json, type);

        long uploadDateTimestamp = updatedFileUpload.getUploadDate().getTime();
        map.put("uploadDate", uploadDateTimestamp);

        String fileId = updatedFileUpload.getFileId();

        r.db("sharepulse").table("file_uploads")
                .filter(r.hashMap("fileId", fileId))
                .update(map)
                .run(connection);
    }

    public void deleteFileUploadByFileId(String fileId) {
        r.db("sharepulse").table("file_uploads")
                .filter(r.hashMap("fileId", fileId))
                .delete()
                .run(connection);
    }

    // This query filters all file uploads that are older than 24 hours from the file_uplaods table (not the expired_file_uploads table)
    public List<FileUpload> findAllExpiredActiveFileUploads() {

        long timestamp = getOneMinuteAgoTimestamp();

        return r.db("sharepulse").table("file_uploads")
                .filter(row -> row.g("uploadDate").lt(timestamp))
                .run(connection, FileUpload.class)
                .toList();
    }

    public List<FileUpload> findAll() {
        return r.db("sharepulse").table("file_uploads")
                .run(connection, FileUpload.class)
                .toList();
    }

    private long get24HoursAgoTimestamp() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.HOUR, -24);
        return calendar.getTimeInMillis() / 1000;
    }

    private long getOneMinuteAgoTimestamp() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MINUTE, -1); // Subtract 1 minute
        return calendar.getTimeInMillis() / 1000; // Convert milliseconds to seconds (Unix timestamp)
    }
}
