package de.w665.sharepulse.rest.mappings;

import de.w665.sharepulse.exception.NoDownloadPermissionException;
import de.w665.sharepulse.model.FileUpload;
import de.w665.sharepulse.rest.ApiRestController;
import de.w665.sharepulse.service.FileSecurityService;
import de.w665.sharepulse.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.ErrorResponse;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

@Slf4j
@RestController
public class Download extends ApiRestController {

    private final FileService fileService;
    private final FileSecurityService fileSecurityService;

    @Autowired
    public Download(FileService fileService, FileSecurityService fileSecurityService) {
        this.fileService = fileService;
        this.fileSecurityService = fileSecurityService;
    }

    @GetMapping("/download")
    public ResponseEntity<Object> download(@RequestParam String fileId,
                                           @RequestParam(value = "password", required = false) String password) throws IOException {


        Optional<FileUpload> optionalFileUpload = fileService.getFileUploadByFileId(fileId);
        FileUpload fileUpload = optionalFileUpload.orElse(null);
        if(optionalFileUpload.isEmpty()) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        try {
            fileSecurityService.verifyDownloadPermission(fileUpload, password);
        } catch (NoDownloadPermissionException e) {
            log.debug("No download permission for file: " + fileId);
            ErrorResponse errorResponse = ErrorResponse.builder(e, HttpStatus.FORBIDDEN, e.getMessage())
                    .build();
            return new ResponseEntity<>(errorResponse, HttpStatus.FORBIDDEN);
        }

        fileService.updateDownloadCount(fileUpload);

        File file = fileService.getFileById(fileId);
        InputStreamResource resource = new InputStreamResource(new FileInputStream(file));


        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + fileUpload.getFileName() + "\"");
        headers.add(HttpHeaders.CACHE_CONTROL, "no-cache, no-store, must-revalidate");
        headers.add(HttpHeaders.PRAGMA, "no-cache");
        headers.add(HttpHeaders.EXPIRES, "0");
        headers.add(HttpHeaders.ACCESS_CONTROL_EXPOSE_HEADERS, "Content-Disposition");

        return ResponseEntity.ok()
                .headers(headers)
                .contentLength(file.length())
                .contentType(MediaType.APPLICATION_OCTET_STREAM)
                .body(resource);
    }

    @GetMapping("/download-info")
    public ResponseEntity<Object> getDownloadInfo(@RequestParam String fileId) {
        Optional<FileUpload> optionalFileUpload = fileService.getFileUploadByFileId(fileId);
        FileUpload fileUpload = optionalFileUpload.orElse(null);
        if(optionalFileUpload.isEmpty()) {
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }

        boolean downloadable = !fileUpload.isSingleDownload() || fileUpload.getDownloadCount() == 0;

        Map<String, Object> response = new HashMap<>();
        response.put("fileId", fileUpload.getFileId());
        response.put("passwordProtected", fileUpload.isPasswordProtected());
        response.put("singleDownload", fileUpload.isSingleDownload());
        response.put("downloadable", downloadable);

        return new ResponseEntity<>(response, HttpStatus.OK);
    }
}
