package de.w665.sharepulse.service;

import de.w665.sharepulse.db.repo.FileUploadRepository;
import de.w665.sharepulse.model.FileUpload;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.http.fileupload.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.Date;
import java.util.Optional;

@Slf4j
@Service
public class FileService {

    private final FileIdService fileIdService;
    private final FileSecurityService fileSecurityService;
    private final FileUploadRepository fileUploadRepository;

    @Value("${sharepulse.temp-filestore-path}")
    private String tempDirPath;

    @Value("${sharepulse.auto-reset-on-startup}")
    private boolean autoResetOnStartup;

    @Autowired
    public FileService(FileIdService fileIdService, FileSecurityService fileSecurityService, FileUploadRepository fileUploadRepository) {
        this.fileIdService = fileIdService;
        this.fileSecurityService = fileSecurityService;
        this.fileUploadRepository = fileUploadRepository;
    }

    @PostConstruct
    public void createFolder() {
        Path path = Paths.get(getTempDirPath());
        log.debug("Checking temp file directory " + path.getFileName());
        try {
            if (!Files.exists(path)) {
                Files.createDirectory(path);
                log.debug("Directory created");
            } else {
                log.debug("Directory already exists.");
                if(autoResetOnStartup) {
                    log.debug("Clearing content...");
                    FileUtils.cleanDirectory(new File(getTempDirPath()));
                    log.debug("Directory content cleared.");
                }

            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public FileUpload processUploadedFile(MultipartFile file, String uploaderIp, boolean passwordProtected, boolean singleDownload, String fileDescription) {

        String fileId = fileIdService.generateNewUniqueId();

        String password = "";
        if (passwordProtected) {
            password = fileSecurityService.generateFilePassword();
        }

        /*String encodedFileName = "";
        try {
            encodedFileName = URLEncoder.encode(file.getOriginalFilename(), StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException e) {
            log.error("Error encoding file name: " + e.getMessage());
            return null;
        }*/

        FileUpload fileUpload = FileUpload.builder()
                .fileId(fileId)
                .fileName(file.getOriginalFilename())
                .fileSize(file.getSize())
                .singleDownload(singleDownload)
                .disabled(false)
                .uploadDate(new Date())
                .uploadedByIpAddress(uploaderIp)
                .downloadCount(0)
                .fileDescription(fileDescription)
                .passwordProtected(passwordProtected)
                .downloadPassword(password)
                .build();

        fileUploadRepository.insertFileUpload(fileUpload);


        Path path = Paths.get(getTempDirPath() + File.separator + fileId);
        try (InputStream inputStream = file.getInputStream()) {
            Files.copy(inputStream, path, StandardCopyOption.REPLACE_EXISTING);
            log.debug("File " + file.getOriginalFilename() + " written to " + path.getFileName());
            return fileUpload;
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public File getFileById(String fileId) {
        return new File(getTempDirPath() + File.separator + fileId);
    }

    public Optional<FileUpload> getFileUploadByFileId(String fileId) {
        return fileUploadRepository.retrieveFileUploadByFileId(fileId);
    }

    public long updateDownloadCount(FileUpload fileUpload) {
        fileUpload.setDownloadCount(fileUpload.getDownloadCount() + 1);
        fileUploadRepository.updateFileUpload(fileUpload);
        return fileUpload.getDownloadCount();
    }

    public void deleteFile(FileUpload fileUpload) {
        fileUploadRepository.deleteFileUploadByFileId(fileUpload.getFileId());
        Path path = Paths.get(getTempDirPath() + File.separator + fileUpload.getFileId());
        try {
            Files.delete(path);
            log.debug("File " + fileUpload.getFileId() + " deleted from disk.");
        } catch (IOException e) {
            log.error("Error deleting file " + fileUpload.getFileId() + " from disk: " + e.getMessage());
        }
    }

    private String getTempDirPath() {
        return System.getProperty("user.dir") + tempDirPath;
    }


}
